unit logodemu;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, ExtCtrls,
  LogoAllU;

type

  TPreviewForm = class(TForm)
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    TDM                  : TDevMode;
    TFLP                 : TFullLogPalette;
    StartW, StartH       : Integer;
    StartSlot            : Integer;
    ImageLeft, ImageTop  : Integer;
    PaletteH, OldPH      : hPalette;
    TheBitmapH, OldBH    : HBitmap;
    TheBitmapDC          : hDC;
    FullScreen           : Boolean;
    procedure CreateParams(VAR Params: TCreateParams); override;
  public
    { Public declarations }
    procedure SetFullScreen(Value : Boolean);
    procedure SetBitmap(const S : String);
  end;

var
  PreviewForm: TPreviewForm;

// ChangeDisplaySettings function is mis-declared in WINDOWS.PAS.
//   It does not permit NIL for lpDevMode parameter.
function ChangeDisplaySettings(lpDevMode: PDeviceMode; dwFlags:
  DWORD): Longint; stdcall;external 'user32.dll'
  name 'ChangeDisplaySettingsA';

implementation

{$R *.DFM}

procedure TPreviewForm.CreateParams(VAR Params: TCreateParams);
// Delphi rejects a maximized WindowState when the border style is
//   bsNone. We force it in CreateParams.
begin
  Inherited CreateParams(Params);
  WITH Params DO Style := WS_POPUP OR WS_MAXIMIZE;
end;

procedure TPreviewForm.FormCreate(Sender: TObject);
begin
  ImageLeft  := (Screen.Width - 640) DIV 2;
  ImageTop   := (Screen.Height - 400) DIV 2;
  FullScreen := True;
end;

procedure TPreviewForm.SetFullScreen(Value : Boolean);
begin
  FullScreen := Value;
end;

procedure TPreviewForm.SetBitmap(const S : String);
VAR
  FStream : TFileStream;
  MStream : TMemoryStream;
  N       : Integer;
  BFH     : TBitmapFileHeader;
  BFI     : TBitmapFullInfo;
begin
  FStream := TFileStream.Create(S, fmOpenRead);
  MStream := TMemoryStream.Create;
  try
    FStream.ReadBuffer(BFH, SizeOf(BFH));
    FStream.ReadBuffer(BFI.BIH, SizeOf(BFI.BIH));
    IF (BFH.bfSize              = 129078)  AND
       (BFH.bfOffBits           = 1078)    AND
       (BFI.BIH.biWidth         = 320)     AND
       (BFI.BIH.biHeight        = 400)     AND
       (BFI.BIH.biSizeImage     = 320*400) AND
       ((BFI.BIH.biClrUsed      = 256)       OR
       (BFI.BIH.biClrUsed       = 0))      THEN
      begin
        // Some animated logo bitmaps produced by other programs
        // have biClrImportant > 256! E.g. VISION.SYS
        StartSlot := BFI.BIH.biClrImportant AND $FF;
        // StartSlot is 0-based index of first animation color
        FStream.ReadBuffer(BFI.bmiColors, SizeOf(BFI.bmiColors));
        FillChar(TFLP, SizeOf(TFLP), 0);
        WITH TFLP DO
          begin
            PalVersion    := $300;
            PalNumEntries := 256;
            FOR N := 0 TO 255 DO
              begin
                WITH palPalEntry[N], BFI.bmiColors[N] DO
                  begin
                    peRed   := rgbRed;
                    peBlue  := rgbBlue;
                    peGreen := rgbGreen;
                    peFlags := 0;
                  end;
              end;
            FOR N := StartSlot TO 255 DO
              palPalEntry[N].peFlags := PC_RESERVED;
            // Create and select the palette
            PaletteH := CreatePalette(PLogPalette(@TFLP)^);
            OldPH := SelectPalette(Canvas.Handle, PaletteH, False);
            SetSystemPaletteUse(Canvas.Handle, SYSPAL_NOSTATIC);
            UnrealizeObject(PaletteH);
            RealizePalette(Canvas.Handle);
            // Create the bitmap handle
            MStream.SetSize(320*400);
            FStream.ReadBuffer(MStream.Memory^, 320*400);
            TheBitmapH := CreateDIBitmap(Canvas.Handle,
              PBitmapInfoHeader(@BFI)^, CBM_INIT, MStream.Memory,
              PBitmapInfo(@BFI)^ , DIB_RGB_COLORS);
            TheBitmapDC := CreateCompatibleDC(Canvas.Handle);
            OldBH := SelectObject(TheBitmapDC, TheBitmapH);
          end;
      end;
  finally
    FStream.Free;
    MStream.Free;
  end;
  Timer1.Enabled := True;
  IF NOT FullScreen THEN Exit;
  // get current settings
  StartW           := Screen.Width;
  StartH           := Screen.Height;
  TDM.dmSize       := SizeOf(TDM);
  EnumDisplaySettings(NIL, 0, TDM);
  // set screen to 640x400 (assumes 256 color)
  TDM.dmPelsWidth  := 640;
  TDM.dmPelsHeight := 400;
  TDM.dmFields     := DM_PELSHEIGHT OR DM_PELSWIDTH;
  IF ChangeDisplaySettings(@TDM, 0) = DISP_CHANGE_SUCCESSFUL THEN
    begin
      ImageLeft := 0;
      ImageTop  := 0;
    end;
end;

// fig06.txt begin
procedure TPreviewForm.Timer1Timer(Sender: TObject);
VAR
  N       : Integer;
  PE      : TPaletteEntry;
  hOldPal : hPalette;
begin
  // cycle the palette colors
  PE := TFLP.PalPalEntry[StartSlot];
  FOR N := StartSlot TO 254  DO
    TFLP.PalPalEntry[N] := TFLP.PalPalEntry[N+1];
  TFLP.PalPalEntry[255] := PE;
  // animate the palette (only PC_RESERVED items will change)
  hOldPal := SelectPalette(Canvas.Handle, PaletteH, FALSE);
  AnimatePalette(PaletteH, 0, 256, @TFLP.palPalEntry);
  RealizePalette(Canvas.Handle);
  SelectPalette(Canvas.Handle, hOldPal, TRUE);
end;
// fig06.txt end

procedure TPreviewForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  Close;
end;

procedure TPreviewForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Close;
end;

procedure TPreviewForm.FormPaint(Sender: TObject);
begin
  WITH Canvas DO
    begin
      Brush.Color := clBlack;
      FillRect(BoundsRect);
      StretchBlt(Handle, ImageLeft, ImageTop, 640, 400, TheBitmapDC,
        0, 0, 320, 400, SrcCopy);
    end;
end;

procedure TPreviewForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SelectPalette(Canvas.Handle, OldPH, TRUE);
  DeleteObject(PaletteH);
  SelectObject(TheBitmapDC, OldBH);
  DeleteObject(TheBitmapH);
  DeleteDC(TheBitmapDC);
  IF FullScreen THEN
    begin
      TDM.dmPelsWidth  := StartW;
      TDM.dmPelsHeight := StartH;
      TDM.dmFields     := DM_PELSHEIGHT OR DM_PELSWIDTH;
      ChangeDisplaySettings(@TDM, 0);
    end;
  SetSystemPaletteUse(Canvas.Handle, SYSPAL_STATIC); 
end;

end.
