unit logogenu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, LogoAllU;

type
  GeneratedType = (gtBroc, gtCirc, gtDiag, gtDiam, gtElli, gtFuzz,
    gtHalf, gtHorz,
    gtHwav, gtMand, gtMan2, gtMoir, gtRand, gtRect, gtSine, gtSpin, gtSpir,
    gtSqua, gtVert,
    gtVwav);
  GradType = (gtAB, gtABA, gtABC, gtABCA);

  TGenForm = class(TForm)
    Panel1: TPanel;
    pb1: TPaintBox;
    pb2: TPaintBox;
    pb3: TPaintBox;
    rgShape: TRadioGroup;
    rgDirection: TRadioGroup;
    rgGradient: TRadioGroup;
    rgColors: TRadioGroup;
    btnOK: TBitBtn;
    rgWidth: TRadioGroup;
    btnCancel: TBitBtn;
    ColorDialog1: TColorDialog;
    PopupMenu1: TPopupMenu;
    A1: TMenuItem;
    B1: TMenuItem;
    C1: TMenuItem;
    procedure btnAClick(Sender: TObject);
    procedure pb1Paint(Sender: TObject);
    procedure pb1DblClick(Sender: TObject);
    procedure A1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    GenFilename : String;
  end;

var
  GenForm: TGenForm;

implementation
USES IniFiles;

{$R *.DFM}

procedure TGenForm.btnAClick(Sender: TObject);
VAR pb : TPaintBox;
begin
  pb := TPaintBox((Sender AS TBitBtn).Tag);
  WITH ColorDialog1 DO
    begin
      Color := pb.Color;
      IF Execute THEN
        pb.Color := Color;
    end;
end;

procedure TGenForm.pb1Paint(Sender: TObject);
CONST buff : ARRAY[0..2] OF Char = '&A'#0;
VAR
  R : TRect;
  vR, vG, vB : Integer;
begin
  WITH Sender AS TPaintBox DO
    begin
      R := Rect(0, 0, Width, Height);
      Canvas.Brush.Color := Color;
      Canvas.Pen.Color := clBlack;
      Canvas.Rectangle(0, 0, Width, Height);
      vR := GetRValue(Color);
      vG := GetGValue(Color);
      vB := GetBValue(Color);
      IF (vR+vG+vB) DIV 3 > 64 THEN
        Font.Color := clBlack
      ELSE Font.Color := clWhite;
      Buff[1] := Char(Tag);
      SetBkMode(Canvas.Handle, TRANSPARENT);
      DrawText(Canvas.Handle, Buff, 2, R,
        DT_CENTER OR DT_VCENTER OR DT_SINGLELINE);
    end;
end;

procedure TGenForm.pb1DblClick(Sender: TObject);
begin
  WITH ColorDialog1 DO
    begin
      Color := (Sender AS TPaintBox).Color;
      IF Execute THEN
        (Sender AS TPaintBox).Color := Color;
    end;
end;

procedure TGenForm.A1Click(Sender: TObject);
begin
  pb1DblClick(TPaintBox((Sender AS TMenuItem).Tag));
end;

procedure TGenForm.FormCreate(Sender: TObject);
begin
  A1.Tag := Integer(pb1);
  B1.Tag := Integer(pb2);
  C1.Tag := Integer(pb3);
  Randomize;
  PositionFromIniFile(Self);
  WITH TIniFile.Create(IniName) DO
    try
      rgShape.ItemIndex     := ReadInteger('GenForm', 'Shape', 0);
      rgDirection.ItemIndex := ReadInteger('GenForm', 'Direction',0);
      rgGradient.ItemIndex  := ReadInteger('GenForm', 'Gradient', 2);
      rgColors.ItemIndex    := ReadInteger('GenForm', 'Colors', 8);
      rgWidth.ItemIndex     := ReadInteger('GenForm', 'Width', 1);
      pb1.Color := ReadInteger('GenForm', 'Color1', clRed);
      pb2.Color := ReadInteger('GenForm', 'Color2', clBlue);
      pb3.Color := ReadInteger('GenForm', 'Color3', clLime);
    finally
      Free;
    end;
end;

procedure TGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PositionToIniFile(Self);
  WITH TIniFile.Create(IniName) DO
    try
      WriteInteger('GenForm', 'Shape', rgShape.ItemIndex);
      WriteInteger('GenForm', 'Direction',rgDirection.ItemIndex);
      WriteInteger('GenForm', 'Gradient', rgGradient.ItemIndex);
      WriteInteger('GenForm', 'Colors', rgColors.ItemIndex);
      WriteInteger('GenForm', 'Width', rgWidth.ItemIndex);
      WriteInteger('GenForm', 'Color1', pb1.Color);
      WriteInteger('GenForm', 'Color2', pb2.Color);
      WriteInteger('GenForm', 'Color3', pb3.Color);
    finally
      Free;
    end;
end;

// FIG05.TXT begins
function ColorFunc(vRow, vCol : Integer; vGenT : GeneratedType;
  vIsIn: Boolean; vColors, vWid : Integer) : Integer;

  function FPUArcTan(Y, X : Extended) : Extended;
  asm
    FLD Y;
    FLD X;
    FPATAN;
    FWAIT;
  end;

  function MandelbrotFunc(vL, vT, vW, vH : Extended) : Integer;
  VAR FixedX, FixedY, X, Y, NuX, NuY : Extended;
  begin
    FixedX := (vCol*vW) + vL;
    FixedY := (vRow*vH) + vT;
    X := 0;
    Y := 0;
    Result := 0;
    REPEAT
      nuX := Sqr(X)-Sqr(Y)-FixedX;
      nuY := 2*X*Y-FixedY;
      X := NuX;
      Y := NuY;
      Inc(Result);
    UNTIL (Result = vColors) OR (Sqr(X)+Sqr(Y)>= 4);
    Result := vColors-Result;
  end;

  function SpiralFunc(vR, vC : Integer) : Integer;
  VAR R, Theta : Extended;
  begin
    R      := Sqrt(Sqr(vR-200) + Sqr(2*vCol-320));
    Theta  := FPUArcTan(vR-200, 2*vC-320)+pi;
    Result := (Round(R+vColors*vWid*Theta/Pi) DIV vWid) MOD vColors;
  end;

begin
  Result := 0;
  CASE vGenT OF
    gtBroc : Result := (((vCol*2*Sqr(vRow)) DIV vWid) MOD vColors);
    gtCirc : Result := (Round(Sqrt(Sqr(vCol*2-320)+Sqr(vRow-200)))
               DIV vWid) MOD vColors;
    gtDiag : Result := (((vCol*2+vRow) DIV vWid) MOD vColors);
    gtDiam : Result := ((Abs(2*vCol-320) + Abs(vRow-200)) DIV vWid)
               MOD vColors;
    gtElli : Result := (Round(Sqrt(Sqr(vCol-160)+Sqr(vRow-200))) DIV
               vWid) MOD vColors;
    gtFuzz : Result := (Round((Sqrt(Sqr(vCol*2-320)+Sqr(vRow-200)))+
               Random(Sqr(vWid))) DIV vWid) MOD vColors;
    gtHalf : Result := (Round(Sqrt(Sqr(vCol*2-320)+Sqr(vRow-400)))
               DIV vWid) MOD vColors;
    gtHorz : Result := ((vRow DIV vWid) MOD vColors);
    gtHwav : Result := (((vCol*2+Sqr(vRow)) DIV vWid) MOD vColors);
    gtMand : Result := MandelbrotFunc(-0.8, -1.25, 2.8/320, 2.5/400);
    gtMan2 : Result := MandelbrotFunc(1, -0.5, 0.3/320, 0.3/400);
    gtMoir : Result := ((Round(vCol*2*Sqrt(vRow)) DIV vWid) MOD vColors);
    gtRand : Result := Random(vColors);
    gtRect : IF (Abs(2*vCol-320)*5) DIV 8 > Abs(vRow-200) THEN
      Result := (((Abs(2*vCol-320)*5) DIV 8) DIV vWid) MOD vColors
    ELSE Result := (Abs(vRow-200) DIV vWid) MOD vColors;
    gtSine : Result := ((vRow + 400 +
      Round(200*(Sin(((2*vCol-320)/320)*pi)))) DIV vWid) MOD vColors;
    gtSpin : Result := Round(vColors*vWid*(FPUArcTan(vRow-200,
      2*(vCol-160))+pi)/Pi) MOD vColors;
    gtSpir : Result := SpiralFunc(vRow, vCol);
    gtSqua : IF Abs(2*vCol-320) > Abs(vRow-200) THEN
      Result := (Abs(2*vCol-320) DIV vWid) MOD vColors
    ELSE Result := (Abs(vRow-200) DIV vWid) MOD vColors;
    gtVert : Result := ((vCol DIV vWid) MOD vColors);
    gtVwav : Result := (((Sqr(vCol)*2+vRow) DIV vWid) MOD vColors);
  end;
  IF vIsIn THEN
    Result := Result + 1
  ELSE Result := vColors - Result;
end;
// FIG05.TXT ends

procedure GenerateBitmap(CONST S : String; GenT: GeneratedType;
  IsIn : Boolean; GradT : GradType; Colors, LineWid : Integer;
  cA, cB, cC : TColorRef);
const
  LowStat : ARRAY[0..9] OF TRGBQuad =
   ((rgbBlue:$00; rgbGreen:$00; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$00; rgbRed:$80;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$80; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$80; rgbRed:$80;rgbReserved:$00),
    (rgbBlue:$80; rgbGreen:$00; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$80; rgbGreen:$00; rgbRed:$80;rgbReserved:$00),
    (rgbBlue:$80; rgbGreen:$80; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$C0; rgbGreen:$C0; rgbRed:$C0;rgbReserved:$00),
    (rgbBlue:$C0; rgbGreen:$DC; rgbRed:$C0;rgbReserved:$00),
    (rgbBlue:$F0; rgbGreen:$CA; rgbRed:$A6;rgbReserved:$00));
  HighStat : ARRAY[0..9] OF TRGBQuad =
   ((rgbBlue:$F0; rgbGreen:$FB; rgbRed:$FF;rgbReserved:$00),
    (rgbBlue:$A4; rgbGreen:$A0; rgbRed:$A0;rgbReserved:$00),
    (rgbBlue:$80; rgbGreen:$80; rgbRed:$80;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$00; rgbRed:$FF;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$FF; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$00; rgbGreen:$FF; rgbRed:$FF;rgbReserved:$00),
    (rgbBlue:$FF; rgbGreen:$00; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$FF; rgbGreen:$00; rgbRed:$FF;rgbReserved:$00),
    (rgbBlue:$FF; rgbGreen:$FF; rgbRed:$00;rgbReserved:$00),
    (rgbBlue:$FF; rgbGreen:$FF; rgbRed:$FF;rgbReserved:$00));
TYPE
  TFilePaletteType = ARRAY[0..255] OF TRGBQuad;
VAR
  F : File;
  vBFH : TBitmapFileHeader;
  vBIH : TBitmapInfoHeader;
  vPal : TFilePaletteType;
  Pixelrow : ARRAY[0..319] OF Byte;
  ro, co : Integer;

  procedure Gradiate(idxFm, idxTo : Integer);
  VAR
    gR, gB, gG : Double;
    sR, sB, sG, X : Integer;
  begin
    IF idxFm=idxTo THEN Exit;
    sR := vPal[idxFm].rgbRed;
    sG := vPal[idxFm].rgbGreen;
    sB := vPal[idxFm].rgbBlue;
    gR := (vPal[idxTo].rgbRed - sR)   / (idxTo-idxFm);
    gG := (vPal[idxTo].rgbGreen - sG) / (idxTo-idxFm);
    gB := (vPal[idxTo].rgbBlue - sB)  / (idxTo-idxFm);
    FOR X := IdxFm+1 TO idxTo-1 DO
      WITH vPal[X] DO
        begin
          rgbRed   := sR + Round((X-idxFm)*gR);
          rgbGreen := sG + Round((X-idxFm)*gG);
          rgbBlue  := sB + Round((X-idxFm)*gB);
        end;
  end;

begin
  FillChar(vBFH, SizeOf(vBFH), 0);
  FillChar(vBIH, SizeOf(vBIH), 0);
  FillChar(vPal, SizeOf(vPal), 0);
  WITH vBFH DO
    begin
      bfType := $4D42;
      bfOffBits := SizeOf(TBitmapFileHeader) +
        SizeOf(TBitmapInfoHeader) +SizeOf(TFilePaletteType);
      bfSize    := 320*400 + bfOffBits;
    end;
  WITH vBIH DO
    begin
      biSize          := SizeOf(TBitmapInfoHeader);
      biWidth         := 320;
      biHeight        := 400;
      biPlanes        := 1;
      biBitCount      := 8;
      biCompression   := BI_RGB;
      biSizeImage     := 0;
      biXPelsPerMeter := 3780;
      biYPelsPerMeter := 3790;
      biClrUsed       := 0;
      biClrImportant  := 0;
    end;
  Move(LowStat, vPal[0], 10*SizeOf(TRGBQuad));
  Move(HighStat, vPal[246], 10*SizeOf(TRGBQuad));
  WITH vPal[10] DO // set color 1
    begin
      rgbRed   := GetRValue(cA);
      rgbGreen := GetGValue(cA);
      rgbBlue  := GetBValue(cA);
    end;
  CASE GradT OF
    gtAB : begin
      WITH vPal[Colors+9] DO // set color 2
        begin
          rgbRed   := GetRValue(cB);
          rgbGreen := GetGValue(cB);
          rgbBlue  := GetBValue(cB);
        end;
      Gradiate(10, Colors+9);
    end;
    gtABA : begin
      WITH vPal[(Colors DIV 2)+9] DO // set color 2
        begin
          rgbRed   := GetRValue(cB);
          rgbGreen := GetGValue(cB);
          rgbBlue  := GetBValue(cB);
        end;
      WITH vPal[Colors+10] DO // set color 1 again
        begin
          rgbRed   := GetRValue(cA);
          rgbGreen := GetGValue(cA);
          rgbBlue  := GetBValue(cA);
        end;
      Gradiate(10, (Colors DIV 2)+9);
      Gradiate((Colors DIV 2)+9, Colors+10);
    end;
    gtABC : begin
      WITH vPal[(Colors DIV 2)+9] DO // set color 2
        begin
          rgbRed   := GetRValue(cB);
          rgbGreen := GetGValue(cB);
          rgbBlue  := GetBValue(cB);
        end;
      WITH vPal[Colors+9] DO        // set color 3
        begin
          rgbRed   := GetRValue(cC);
          rgbGreen := GetGValue(cC);
          rgbBlue  := GetBValue(cC);
        end;
      Gradiate(10, (Colors DIV 2)+9);
      Gradiate((Colors DIV 2)+9, Colors+9);
    end;
    gtABCA : begin
      WITH vPal[(Colors DIV 3)+10] DO // set color 2
        begin
          rgbRed   := GetRValue(cB);
          rgbGreen := GetGValue(cB);
          rgbBlue  := GetBValue(cB);
        end;
      WITH vPal[2*(Colors DIV 3)+10] DO        // set color 3
        begin
          rgbRed   := GetRValue(cC);
          rgbGreen := GetGValue(cC);
          rgbBlue  := GetBValue(cC);
        end;
      WITH vPal[Colors+10] DO // set color 1 again
        begin
          rgbRed   := GetRValue(cA);
          rgbGreen := GetGValue(cA);
          rgbBlue  := GetBValue(cA);
        end;
      Gradiate(10, (Colors DIV 3)+10);
      Gradiate((Colors DIV 3)+10, 2*(Colors DIV 3)+10);
      Gradiate(2*(Colors DIV 3)+10, Colors+10);
    end;
  END;
  AssignFile(F, S);
  Rewrite(F, 1);
  try
    BlockWrite(F, vBFH, SizeOf(vBFH));
    BlockWrite(F, vBIH, SizeOf(vBIH));
    BlockWrite(F, vPal, SizeOf(vPal));
    Screen.Cursor := crHourglass;
    try
      FOR ro := 0 TO 399 DO
        begin
          FOR co := 0 TO 319 DO
            PixelRow[co] := ColorFunc(ro, co, GenT, IsIn, Colors, LineWid)+9;
          BlockWrite(F, PixelRow, SizeOf(PixelRow));
        end;
    finally
      Screen.Cursor := crDefault;
    end;
  finally
    CloseFile(F);
  end;
end;

procedure TGenForm.btnOKClick(Sender: TObject);
begin
  GenerateBitmap(GenFilename, GeneratedType(rgShape.ItemIndex),
    Boolean(rgDirection.itemIndex), GradType(rgGradient.ItemIndex),
    (rgColors.ItemIndex+1)*4, Succ(rgWidth.ItemIndex), pb1.Color,
    pb2.Color, pb3.Color);
  ModalResult := mrOK;
end;

procedure TGenForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  KeyWasF1 := (Key = VK_F1) AND (Shift=[]);
end;

end.
