unit logomanu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ExtCtrls, StdCtrls, Buttons, ComCtrls,
  loginfou, logoPalu, LogoAllU;

type
  PBitmapStuff = ^TBitmapStuff;
  TBitmapStuff = record
    TheFilename  : String;
    BFH          : TBitmapFileHeader;
    BIH          : TBitmapInfoHeader;
    BIPalette    : TFilePaletteType;
    PixelCount   : TPixelCountType;
    ColorsInUse  : Integer;
    BitmapType   : logoType;
    lblFilename  : TLabel;
    lblSize      : TLabel;
    lblColors    : TLabel;
    btnDetails   : TBitBtn;
    btnResize    : TBitBtn;
    btnCenter    : TBitBtn;
    cbForeground : TCheckBox;
    cbOtherFore  : TCheckBox;
    imgThumbnail : TImage;
  end;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    tsAnimation: TTabSheet;
    tsStatic: TTabSheet;
    tsLogo: TTabSheet;
    lblAnimName: TLabel;
    lblAnimSize: TLabel;
    lblAnimColors: TLabel;
    btnAnimResize: TBitBtn;
    btnAnimChoose: TBitBtn;
    btnAnimDetails: TBitBtn;
    cbAnimForeground: TCheckBox;
    lblStatName: TLabel;
    lblStatSize: TLabel;
    lblStatColors: TLabel;
    btnStatDetails: TBitBtn;
    btnStatResize: TBitBtn;
    btnStatChoose: TBitBtn;
    cbStatForeground: TCheckBox;
    btnFewerColors: TBitBtn;
    edColors: TEdit;
    udColors: TUpDown;
    lblLogoName: TLabel;
    btnCreate: TBitBtn;
    btnLoad: TBitBtn;
    btnPreview: TBitBtn;
    cbFullScreen: TCheckBox;
    tsAbout: TTabSheet;
    Panel1: TPanel;
    imgAnim: TImage;
    imgStat: TImage;
    imgLogo: TImage;
    Bevel1: TBevel;
    imgAbout: TImage;
    btnPalette: TBitBtn;
    btnHelp: TBitBtn;
    btnAnimCenter: TBitBtn;
    btnStatCenter: TBitBtn;
    btnGenerate: TBitBtn;
    btnLogoDetails: TBitBtn;
    lblLogoColors: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure PreviewClick(Sender: TObject);
    procedure btnChooseClick(Sender: TObject);
    procedure btnAnimClick(Sender: TObject);
    procedure btnResizeClick(Sender: TObject);
    procedure cbStatForegroundClick(Sender: TObject);
    procedure btnFewerColorsClick(Sender: TObject);
    procedure btnCreateClick(Sender: TObject);
    procedure btnLoadClick(Sender: TObject);
    procedure btnPaletteClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PageControl1Change(Sender: TObject);
    procedure btnAnimCenterClick(Sender: TObject);
    procedure btnGenerateClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    AnimBitmapStuff,
    StatBitmapStuff,
    LogoBitmapStuff : TBitmapStuff;
    CanPreview      : Boolean;
    procedure ReadAndRateBitmap(const S : String; VAR vBS :
      TBitmapStuff);
    procedure UpdateBitmapStuff(VAR BS : TBitmapStuff);
    procedure CountThePixelsIn(VAR BS : TBitmapStuff);
    procedure ProcrustesBed(VAR BS: TBitmapStuff;
      const OutName: String);
    procedure SaveCentered(VAR BS : TBitmapStuff;
      const OutName : String);
    function GetTransColorSlot(const S : String) : Integer;
    procedure SaveWithReducedColors(VAR BS : TBitmapStuff;
      const S : String; ColorsToUse : Integer; IsFore : Boolean);
    procedure CreateLogoFile(const S : String);
    function SameFile(const S1, S2 : String) : Boolean;
    procedure ErrorMsg(const S : String);
    function AppOnHelp(Command: Word; Data: Longint;
      VAR CallHelp : Boolean): Boolean;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation
uses logodemu, logomodu, logogenu, IniFiles;
{$R *.DFM}
{$I HELPCTX.PAS}

//******** helper methods
procedure TMainForm.ErrorMsg(const S : String);
begin
  MessageBox(Handle,PChar(S),'LogoMania Error',MB_OK OR MB_ICONHAND);
end;

function TMainForm.SameFile(const S1, S2 : String) : Boolean;
VAR
  H1, H2 : THandle;
  I1, I2 : TByHandleFileInformation;
begin
  Result := False;
  H1 := CreateFile(PChar(S1), GENERIC_READ, FILE_SHARE_READ,
    nil, OPEN_EXISTING, 0, 0);
  IF H1 = INVALID_HANDLE_VALUE THEN Exit;
  try
    H2 := CreateFile(PChar(S2), GENERIC_READ, FILE_SHARE_READ,
      nil, OPEN_EXISTING, 0, 0);
    IF H2 = INVALID_HANDLE_VALUE THEN Exit;
    try
      IF NOT GetFileInformationByHandle(H1, I1) THEN Exit;
      IF NOT GetFileInformationByHandle(H2, I2) THEN Exit;
      Result := (I1.dwVolumeSerialNumber=I2.dwVolumeSerialNumber) AND
                (I1.nFileIndexHigh=I2.nFileIndexHigh) AND
                (I1.nFileIndexLow=I2.nFileIndexLow);
    finally
      CloseHandle(H2);
    end;
  finally
    CloseHandle(H1);
  end;
end;

// FIG04.TXT begins
procedure TMainForm.ProcrustesBed(VAR BS : TBitmapStuff;
  const OutName : String);
// Tile or chop wrong-size bitmap to LogoMania's desired size
VAR
  TempFilename             : String;
  HalveWidth, DoubleHeight : Boolean;
  InnF, OutF               : File;
  OutBFH                   : TBitmapFileHeader;
  OutBIH                   : TBitmapInfoHeader;
  PixelsInn                : ARRAY[0..639] OF Byte;
  PixelsOut                : ARRAY[0..319] OF Byte;
  InLineWidth, InReadWidth : Integer;
  InnCol, OutCol           : Integer;
  InnRow, OutRow           : Integer;
begin
  OutBFH := BS.BFH;
  WITH OutBFH DO
    begin
      bfOffBits := SizeOf(TBitmapFileHeader) +
        SizeOf(TBitmapInfoHeader) +SizeOf(TFilePaletteType);
      bfSize    := 320*400 + bfOffBits;
    end;
  OutBIH := BS.BIH;
  WITH OutBIH DO
    begin
      biWidth         := 320;
      biHeight        := 400;
      biSizeImage     := 0;
      biXPelsPerMeter := 3790;
      biYPelsPerMeter := 3780;
      biClrUsed       := 0;
      biClrImportant  := 0;
    end;
  IF (BS.BIH.biWidth=320) AND (BS.BIH.biHeight=400) THEN
    begin
      DoubleHeight := False;
      HalveWidth   := False;
    end
  ELSE
    begin
      DoubleHeight := (BS.BIH.biWidth <= 320) AND
                      (BS.BIH.biHeight <= 200);
      HalveWidth := NOT DoubleHeight;
    end;
  // each line of pixels is 0-padded to 4-byte boundary
  InLineWidth := ((BS.BIH.biWidth + 3) DIV 4) * 4;
  InReadWidth := InLineWidth;
  // There's never a need to read MORE than 640 pixels
  IF InReadWidth>640 THEN InReadWidth := 640;
  TempFilename := Copy(BS.TheFilename, 1, Length(BS.TheFilename)-
    Length(ExtractFileExt(BS.TheFilename))) + '_temp.BMP';
  AssignFile(InnF, BS.TheFilename);
  AssignFile(OutF, TempFilename);
  Reset  (InnF, 1);
  Rewrite(OutF, 1);
  try
    BlockWrite(OutF, OutBFH, SizeOf(OutBFH));
    BlockWrite(OutF, OutBIH, SizeOf(OutBIH));
    BlockWrite(OutF, BS.BIPalette, SizeOf(BS.BIPalette));
    // lines are stored bottom to top; we start at last (top) line so
    // tiling/chopping of image will start from top
    InnRow := BS.BIH.biHeight-1;
    OutRow := 399;
    WHILE OutRow >= 0 DO
      begin
        Seek(InnF, BS.BFH.bfOffBits + InnRow*InLineWidth);
        BlockRead(InnF, PixelsInn, InReadWidth);
        InnCol := 0;
        OutCol := 0;
        WHILE OutCol < 320 DO
          begin
            PixelsOut[OutCol] := PixelsInn[InnCol];
            Inc(OutCol);
            Inc(InnCol);
            // if halving width, skip a pixel
            IF HalveWidth THEN Inc(InnCol);
            // repeat input line if necessary
            IF InnCol >= InReadWidth THEN InnCol := 0;
          end;
        Seek(OutF, OutBFH.bfOffBits + OutRow*320);
        BlockWrite(OutF, PixelsOut, 320);
        Dec(OutRow);
        IF DoubleHeight THEN
          begin
            Seek(OutF, OutBFH.bfOffBits + OutRow*320);
            BlockWrite(OutF, PixelsOut, 320);
            Dec(OutRow);
          end;
        Dec(InnRow);
        // if reached bottom of input, start again
        IF InnRow < 0 THEN InnRow := BS.BIH.biHeight-1;
      end;
  finally
    CloseFile(InnF);
    CloseFile(OutF);
  end;
  IF FileExists(OutName) THEN DeleteFile(OutName);
  RenameFile(TempFilename, OutName);
  BS.BFH := OutBFH;
  BS.BIH := OutBIH;
  BS.TheFilename := OutName;
  BS.BitmapType := lgBMP34;
  UpdateBitmapStuff(BS);
end;
// FIG04.TXT ends

procedure TMainForm.SaveCentered(VAR BS : TBitmapStuff; const OutName : String);
// Center too-small bitmap in LogoMania's desired size
// ONLY used on bitmaps *smaller* than 640x400
VAR
  TempFilename             : String;
  HalveWidth, DoubleHeight : Boolean;
  InnF, OutF                   : File;
  OutBFH                     : TBitmapFileHeader;
  OutBIH                     : TBitmapInfoHeader;
  PixelsInn                : ARRAY[0..639] OF Byte;
  PixelsOut                : ARRAY[0..319] OF Byte;
  StartR, StartC, StopR,
  InLineWidth, InReadWidth,
  InnC, InnR, OutC, OutR   : Integer;
begin
  OutBFH := BS.BFH;
  OutBIH := BS.BIH;
  WITH OutBFH DO
    begin
      bfOffBits := SizeOf(TBitmapFileHeader) +
        SizeOf(TBitmapInfoHeader) +SizeOf(TFilePaletteType);
      bfSize    := 320*400 + bfOffBits;
    end;
  WITH OutBIH DO
    begin
      biWidth         := 320;
      biHeight        := 400;
      biSizeImage     := 0;
      biXPelsPerMeter := 3790;
      biYPelsPerMeter := 3780;
      biClrUsed       := 0;
      biClrImportant  := 0;
    end;
  DoubleHeight := (BS.BIH.biWidth <= 320) AND
                  (BS.BIH.biHeight <= 200);
  HalveWidth := NOT DoubleHeight;
  // each line of pixels is 0-padded to 4-byte boundary
  InLineWidth := ((BS.BIH.biWidth + 3) DIV 4) * 4;
  InReadWidth := InLineWidth;
  // There's never a need to read MORE than 640 pixels
  IF InReadWidth>640 THEN InReadWidth := 640;
  IF DoubleHeight THEN
    begin
      StartR := 200 - BS.BIH.biHeight;
      StopR  := StartR + 2*BS.BIH.biHeight;
      StartC := 160- (BS.BIH.BIWidth DIV 2);
    end
  ELSE
    begin
      StartR := 200 - (BS.BIH.biHeight DIV 2);
      StopR  := StartR + BS.BIH.biHeight;
      StartC := 160 - (BS.BIH.BIWidth DIV 4);
    end;
  TempFilename := Copy(BS.TheFilename, 1, Length(BS.TheFilename)-
    Length(ExtractFileExt(BS.TheFilename))) + '_temp.BMP';
  AssignFile(InnF, BS.TheFilename);
  AssignFile(OutF, TempFilename);
  Reset  (InnF, 1);
  Rewrite(OutF, 1);
  try
    BlockWrite(OutF, OutBFH, SizeOf(OutBFH));
    BlockWrite(OutF, OutBIH, SizeOf(OutBIH));
    BlockWrite(OutF, BS.BIPalette, SizeOf(BS.BIPalette));
    InnR := BS.BIH.biHeight-1;
    OutR := 399;
    // lines are stored bottom to top; we start at last (top) line so
    // tiling/chopping of image will start from top
    WHILE OutR >= 0 DO
      begin
        FillChar(PixelsOut, SizeOf(PixelsOut), 4);
        IF (OutR>=StartR) AND (OutR < StopR) THEN
          begin
            Seek(InnF, BS.BFH.bfOffBits + InnR*InLineWidth);
            BlockRead(InnF, PixelsInn, InReadWidth);
            InnC := 0;
            OutC := StartC;
            WHILE InnC < InReadWidth DO
              begin
                PixelsOut[OutC] := PixelsInn[InnC];
                Inc(OutC);
                Inc(InnC);
                // if halving width, skip a pixel
                IF HalveWidth THEN Inc(InnC);
                // repeat input line if necessary
              end;
            Dec(InnR);
          end;
        Seek(OutF, OutBFH.bfOffBits + OutR*320);
        BlockWrite(OutF, PixelsOut, 320);
        Dec(OutR);
        IF DoubleHeight THEN
          begin
            Seek(OutF, OutBFH.bfOffBits + OutR*320);
            BlockWrite(OutF, PixelsOut, 320);
            Dec(OutR);
          end;
        // if reached bottom of input, start again
      end;
  finally
    CloseFile(InnF);
    CloseFile(OutF);
  end;
  IF FileExists(OutName) THEN DeleteFile(OutName);
  RenameFile(TempFilename, OutName);
  BS.BFH := OutBFH;
  BS.BIH := OutBIH;
  BS.TheFilename := OutName;
  BS.BitmapType := lgBMP34;
  UpdateBitmapStuff(BS);
end;

procedure TMainForm.CreateLogoFile(const S : String);
VAR
  F1, F2, F3   : File;
  TempFilename : String;
  N, M, Transp : Integer;
  HasStat,
  AnimIsFore   : Boolean;
  AnimTrans,
  StatTrans    : ARRAY[0..255] OF Byte;
  PixelRowA,
  PixelRowS,
  PixelRowL    : ARRAY[0..319] OF Byte;
  co, ro       : Integer;
begin
  WITH logoBitmapStuff DO
    begin
      BFH := animBitmapStuff.BFH;
      BIH := animBitmapStuff.BIH;
      FillChar(BIPalette, SizeOf(BIPalette), 0);
      FillChar(PixelCount, SizeOf(PixelCount), 0);
      ColorsInUse := AnimBitmapStuff.ColorsInUse +
        StatBitmapStuff.ColorsInUse - 1;
    end;
  HasStat := StatBitmapStuff.BitmapType <> lgNone;
  AnimIsFore := cbAnimForeground.Checked;
  IF AnimIsFore THEN
    Transp := GetTransColorSlot(animBitmapStuff.TheFileName)
  ELSE IF NOT HasStat THEN Transp := -1
  ELSE Transp := GetTransColorSlot(statBitmapStuff.TheFileName);
  // pack animated bitmap's palette into top
  FillChar(AnimTrans, SizeOf(AnimTrans), 0);
  M := 255; // use topmost color
  FOR N := 255 DOWNTO 0 DO
    IF (animBitmapStuff.PixelCount[N] > 0) AND
       ((NOT AnimIsFore) OR (N<>Transp)) THEN
      begin
        logoBitmapStuff.BIPalette[M].rgbRed :=
          animBitmapStuff.BIPalette[N].rgbRed;
        logoBitmapStuff.BIPalette[M].rgbGreen :=
          animBitmapStuff.BIPalette[N].rgbGreen;
        logoBitmapStuff.BIPalette[M].rgbBlue :=
          animBitmapStuff.BIPalette[N].rgbBlue;
        AnimTrans[N] := M;
        Dec(M);
      end;
  IF AnimIsFore AND (NOT HasStat) THEN
    begin
      logoBitmapStuff.BIPalette[1].rgbRed :=
        animBitmapStuff.BIPalette[Transp].rgbRed;
      logoBitmapStuff.BIPalette[1].rgbGreen :=
        animBitmapStuff.BIPalette[Transp].rgbGreen;
      logoBitmapStuff.BIPalette[1].rgbBlue :=
        animBitmapStuff.BIPalette[Transp].rgbBlue;
      AnimTrans[Transp] := 1;
    end;
  WITH logoBitmapStuff.BIH DO
    begin
      biClrUsed       := 0;
      // set biClrImportant to 0-based index of first animation color
      biClrImportant  := M+1;
      // Fix problem with ALL-TRANSPARENT animation bitmap //12/9
      IF (M=255) AND AnimIsFore THEN biClrImportant := 255;
      biSizeImage     := 320*400;
      biXPelsPerMeter := 3790;
      biYPelsPerMeter := 3780;
    end;
  // pack static bitmaps palette into bottom
  FillChar(StatTrans, SizeOf(StatTrans), 0);
  M := 0;
  FOR N := 0 TO 255 DO
    IF (StatBitmapStuff.PixelCount[N] > 0) AND
       ((AnimIsFore) OR (N<>Transp)) THEN
      begin
        logoBitmapStuff.BIPalette[M].rgbRed :=
          StatBitmapStuff.BIPalette[N].rgbRed;
        logoBitmapStuff.BIPalette[M].rgbGreen :=
          StatBitmapStuff.BIPalette[N].rgbGreen;
        logoBitmapStuff.BIPalette[M].rgbBlue :=
          StatBitmapStuff.BIPalette[N].rgbBlue;
        StatTrans[N] := M;
        Inc(M);
      end;
  TempFilename := animBitmapStuff.TheFilename;
  TempFilename := Copy(TempFilename, 1, Length(TempFilename)-
    Length(ExtractFileExt(TempFilename))) + '_temp.BMP';
  IF FileExists(TempFilename) THEN DeleteFile(TempFilename);
  AssignFile(F1, AnimBitmapStuff.TheFilename);
  IF HasStat THEN
    AssignFile(F2, StatBitmapStuff.TheFilename);
  AssignFile(F3, TempFilename);
  Reset(F1, 1);
  IF HasStat THEN Reset(F2, 1);
  Rewrite(F3, 1);
  try
    BlockWrite(F3, logoBitmapStuff.BFH, SizeOf(logoBitmapStuff.BFH));
    BlockWrite(F3, logoBitmapStuff.BIH, SizeOf(logoBitmapStuff.BIH));
    BlockWrite(F3, logoBitmapStuff.BIPalette, SizeOf(logoBitmapStuff.BIPalette));
    Seek(F1, animBitmapStuff.BFH.bfOffBits);
    IF HasStat THEN Seek(F2, StatBitmapStuff.BFH.bfOffBits);
    FOR ro := 0 TO 399 DO
      begin
        BlockRead(F1, PixelRowA, SizeOf(PixelRowA));
        IF HasStat THEN BlockRead(F2, PixelRowS, SizeOf(PixelRowS));
        IF AnimIsFore OR (NOT HasStat) THEN
          begin
            FOR co := 0 TO 319 DO
              IF (PixelRowA[co]=Transp) AND HasStat THEN
                PixelRowL[co] := StatTrans[PixelRowS[co]]
              ELSE PixelRowL[co] := AnimTrans[PixelRowA[co]];
          end
        ELSE
          begin
            FOR co := 0 TO 319 DO
              IF PixelRowS[co]=Transp THEN
                PixelRowL[co] := AnimTrans[PixelRowA[co]]
              ELSE PixelRowL[co] := StatTrans[PixelRowS[co]];
          end;
        BlockWrite(F3, PixelRowL, SizeOf(PixelRowL));
      end;
  finally
    CloseFile(F1);
    IF HasStat THEN CloseFile(F2);
    CloseFile(F3);
  end;
  IF FileExists(S) THEN DeleteFile(S);
  RenameFile(TempFileName, S);
  logoBitmapStuff.TheFilename := S;
  btnLogoDetails.Enabled := True;
  lblLogoName.Caption := ExtractFileName(S);
  lblLogoColors.Caption := Format('Uses %d colors',
    [logoBitmapStuff.ColorsInUse]);
  logoBitmapStuff.BitmapType := lgSYSa;
  imgLogo.Picture.Bitmap.LoadFromFile(logoBitmapStuff.TheFilename);
  btnPreview.Enabled := CanPreview;
end;

procedure TMainForm.CountThePixelsIn(VAR BS : TBitmapStuff);
VAR
  F        : File;
  PixelRow : ARRAY[0..319] OF Byte;
  ro, co   : Integer;
begin
  AssignFile(F, BS.TheFilename);
  Reset(F, 1);
  try
    FillChar(BS.PixelCount, SizeOf(BS.PixelCount), 0);
    Seek(F, BS.BFH.bfOffBits);
    FOR ro := 0 TO 399 DO
      begin
        BlockRead(F, PixelRow, 320);
        FOR co := 0 TO 319 DO Inc(BS.PixelCount[PixelRow[co]]);
      end;
    BS.ColorsInUse := 0;
    FOR ro := 0 TO 255 DO
      IF BS.PixelCount[ro] <> 0 THEN
        Inc(BS.ColorsInUse);
  finally
    CloseFile(F);
  end;
end;

procedure TMainForm.ReadAndRateBitmap(const S : String; VAR vBS : TBitmapStuff);
// Make sure the filename is a valid 256-color bitmap;
// determine whether it's one of Logomania's special types
VAR
  F      : File;
  Actual : Integer;

  function FailMsg(Cond : Boolean; Msg : String) : Boolean;
  begin
    Result := NOT Cond;
    IF Result THEN ErrorMsg(S + ' ' + Msg);
  end;
begin
  vBS.BitmapType  := lgNone;
  AssignFile(F, S);
  try
    Reset(F, 1);
    try
      BlockRead(F, vBS.BFH, SizeOf(vBS.BFH), Actual);
      IF FailMsg(Actual=SizeOf(vBS.BFH),'is not a bitmap file.') THEN Exit;
      IF FailMsg(vBS.BFH.bfType=$4D42, 'is not a bitmap file.') THEN Exit;
      BlockRead(F, vBS.BIH, SizeOf(vBS.BIH), Actual);
      IF FailMsg(Actual=SizeOf(vBS.BIH),'is not a bitmap file.') THEN Exit;
      IF FailMsg(vBS.BIH.biSize=SizeOf(TBitmapInfoHeader), ' may be an '+
        'OS/2-style bitmap file. Only Windows-style bitmaps can be used.')
          THEN Exit;
      IF FailMsg(vBS.BIH.biBitCount=8,
        Format('is a %d-color bitmap. Only 256-color bitmaps '+
        'can be used.', [LongInt(1) SHL vBS.BIH.biBitCount])) THEN Exit;
      IF FailMsg(vBS.BIH.biCompression=BI_RGB, 'is a compressed bitmap. '+
        'Only uncompressed bitmaps can be used') THEN Exit;
      FillChar(vBS.BIPalette, SizeOf(vBS.BIPalette), 0);
      Seek(F, SizeOf(vBS.BFH)+SizeOf(vBS.BIH));
      IF vBS.BIH.biClrUsed > 0 THEN
        BlockRead(F, vBS.BIPalette, SizeOf(TRGBQuad)*vBS.BIH.biClrUsed)
      ELSE BlockRead(F, vBS.BIPalette, SizeOf(TRGBQuad)*256);
      vBS.BitmapType := lgBMP;
      IF (vBS.BIH.biWidth=320) AND (vBS.BIH.biHeight=400) AND
         (vBS.BIH.biPlanes=1) AND (vBS.BIH.biBitCount=8) THEN
        begin
          IF (UpperCase(ExtractFileExt(S))='.SYS') AND
            // VISION.SYS required addition of ...=256
            // it also has clrImportant=258???
            ((vBS.BIH.biclrUsed=0) OR (vBS.BIH.biclrUsed=256)) THEN
            begin
              IF vBS.BIH.biclrImportant>0 THEN
                vBS.BitmapType := lgSYSa
              ELSE vBS.BitmapType := lgSYS;
            end
          ELSE vBS.BitmapType := lgBMP34;
        end;
      vBS.TheFilename := S;
    finally
      CloseFile(F);
    end;
  except
    ON EInOutError DO
      ErrorMsg('Unable to open file '+S);
  end;
end;

procedure TMainForm.UpdateBitmapStuff(VAR BS : TBitmapStuff);
// assumes BS.TheFilename and BS.BitmapType are initialized
  procedure StatColorInfo;
  begin
    btnFewerColors.Enabled := False;
    edColors.Text := '';
    edColors.Enabled := False;
    udColors.Enabled := False;
    btnCreate.Enabled := False;
    IF AnimBitmapStuff.BitmapType > lgBMP THEN
      begin
        IF StatBitmapStuff.BitmapType = lgNone THEN
          begin
            btnCreate.Enabled := True;
          end
        ELSE IF StatBitmapStuff.BitmapType > lgBMP THEN
          begin
            IF AnimBitmapStuff.ColorsInUse+
               StatBitmapStuff.ColorsInUse<=256 THEN
              btnCreate.Enabled := True
            ELSE
              begin
                btnFewerColors.Enabled := True;
                edColors.Enabled := True;
                udColors.Enabled := True;
                udColors.Max := 256-AnimBitmapStuff.ColorsInUse;
                udColors.Position := udColors.Max;
                edColors.Text := IntToStr(udColors.Position);
                lblStatColors.Caption := 'Must reduce colors';
              end;
          end;
      end;
  end;

begin
  IF (BS.TheFilename='') OR (BS.BitmapType=lgNone) THEN
    begin
      FillChar(BS.BFH, SizeOf(BS.BFH), 0);
      FillChar(BS.BIH, SizeOf(BS.BFH), 0);
      FillChar(BS.BIPalette, SizeOf(BS.BIPalette), 0);
      FillChar(BS.PixelCount, SizeOf(BS.PixelCount), 0);
      BS.TheFilename          := '';
      BS.ColorsInUse          := 0;
      BS.BitmapType           := lgNone;
      BS.btnDetails.Enabled   := False;
      BS.btnResize.Enabled    := False;
      BS.btnCenter.Enabled    := False;
      BS.lblFileName.Caption  := '(none)';
      BS.lblSize.Caption      := '';
      BS.lblColors.Caption    := '';
      BS.imgThumbNail.Picture.Bitmap := NIL;
      IF BS.btnDetails=btnAnimDetails THEN
        btnPalette.Enabled := False;
      btnCreate.Enabled    := AnimBitmapStuff.BitmapType > lgBMP;
      btnFewerColors.Enabled := False;
      edColors.Enabled := False;
      udColors.Enabled := False;
    end
  else
    begin
      BS.lblFileName.Caption := ExtractFileName(BS.TheFilename);
      BS.btnDetails.Enabled := True;
      IF BS.BitmapType = lgBMP THEN
        begin
          IF BS.btnDetails=btnAnimDetails THEN
            btnPalette.Enabled := False;
          BS.btnResize.Enabled  := True;
          BS.btnCenter.Enabled  := (BS.BIH.biWidth<640) AND
            (BS.BIH.biHeight < 400);
          BS.lblSize.Caption := Format('%dx%d (must resize)',
            [BS.BIH.biWidth, BS.BIH.biHeight]);
          BS.lblColors.Caption := '';
          BS.ColorsInUse := -1;  
          IF (BS.BIH.biWidth <= 160) AND (BS.BIH.biHeight <= 100) THEN
            begin
              BS.imgThumbnail.Width := BS.BIH.biWidth;
              BS.imgThumbnail.Height := BS.BIH.biHeight;
            end
          ELSE
            begin
              IF BS.BIH.biWidth / BS.BIH.biHeight > 1.6 THEN
                begin
                  BS.imgThumbnail.Width := 160;
                  BS.imgThumbnail.Height := (BS.BIH.biHeight * 160) DIV BS.BIH.biWidth;
                end
              ELSE
                begin
                  BS.imgThumbnail.Height := 100;
                  BS.imgThumbnail.Width := (BS.BIH.biWidth * 100) DIV BS.BIH.biHeight;
                end;
            end;
          BS.imgThumbnail.Picture.Bitmap.LoadFromFile(BS.thefilename);//!
          Exit;
        end;
      CountThePixelsIn(BS);
      BS.lblSize.Caption := Format('%dx%d (%d colors)',
        [BS.BIH.biWidth, BS.BIH.biHeight, BS.ColorsInUse]);
      BS.lblColors.Caption := Format('Uses %d colors',
        [BS.ColorsInUse]);
      StatColorInfo;
      IF BS.btnDetails=btnAnimDetails THEN
        btnPalette.Enabled := True;
      BS.btnResize.Enabled  := False;
      BS.btnCenter.Enabled  := False;
      BS.imgThumbnail.Width := 160;
      BS.imgThumbnail.Height := 100;
      BS.imgThumbnail.Picture.Bitmap.LoadFromFile(BS.TheFilename);
    end;
end;

function TMainForm.GetTransColorSlot(const S : String) : Integer;
VAR
  vBFH : TBitmapFileHeader;
  F    : File;
  B    : Byte;
begin
  AssignFile(F, S);
  Reset(F, 1);
  BlockRead(F, vBFH, SizeOf(vBFH));
  Seek(F, vBFH.bfOffBits);
  BlockRead(F, B, 1);
  Result := B;
  CloseFile(F);
end;

// FIG04b.txt begin
procedure TMainForm.SaveWithReducedColors(VAR BS : TBitmapStuff;
  const S : String; ColorsToUse : Integer; IsFore : Boolean);
TYPE
  PalSortType = record
    OrgIndex, NewIndex         : Byte;
    pstRed, pstGreen, pstBlue  : Byte;
    Pixels                     : Integer;
  end;
VAR
  PalSort             : ARRAY[0..255] OF PalSortType;
  TransColorSlot, N,M,
  SlotMax, SlotsUsed  : Integer;
  F1, F2              : File;
  TransTable          : ARRAY[0..255] OF Byte;
  NuFilePalette       : ARRAY[0..255] OF TRGBQuad;
  PixelRow            : ARRAY[0..319] OF Byte;
  TempFileName        : String;

  function EquColor(A, B : PalSortType) : Boolean;
  begin
    Result := (A.pstRed   = B.pstRed)   AND
      (A.pstGreen = B.pstGreen) AND (A.pstBlue  = B.pstBlue);
  end;

  procedure SortByPrevalence;
  VAR
    X, Y     : Integer;
    Swaptemp : PalSortType;
  begin
    // sort by most common to least - simple selection sort
    FOR X := 255 DOWNTO 2 DO
      FOR Y := 1 TO X-1 DO
        // sort the transparent color (if any) into 2nd slot
        IF (PalSort[X].OrgIndex = TransColorSlot) OR
           (PalSort[X].Pixels > PalSort[Y].Pixels) THEN
          begin
            SwapTemp := PalSort[Y];
            PalSort[Y] := PalSort[X];
            PalSort[X] := SwapTemp;
          end;
  end;

  procedure SortOnOrgIndex;
  VAR
    X, Y     : Integer;
    Swaptemp : PalSortType;
  begin
    // restore original order - simple selection sort
    FOR X := 255 DOWNTO 1 DO
      FOR Y := 0 TO X-1 DO
        IF PalSort[X].OrgIndex < PalSort[Y].OrgIndex THEN
          begin
            SwapTemp   := PalSort[Y];
            PalSort[Y] := PalSort[X];
            PalSort[X] := SwapTemp;
          end;
  end;

  procedure ReduceColors;
  VAR
    X, CloseX, OneDist, ClosestDist : Integer;
    Swaptemp                        : PalSortType;

    function DistFrom(A, B : PalSortType) : Integer;
    begin
      Result := Abs(A.pstRed   - B.pstRed) +
        Abs(A.pstGreen - B.pstGreen) + Abs(A.pstBlue  - B.pstBlue);
    end;

  begin
    WHILE SlotsUsed > SlotMax DO
      begin
        ClosestDist := 3*255;
        CloseX  := SlotsUsed-1;
        // slots are in order of prevalence; from SlotsUsed
        // to 255 they are all zero. Search downward seeking
        // closest color match.
        FOR X := SlotsUsed-2 DOWNTO 0 DO
          begin
            OneDist := DistFrom(PalSort[SlotsUsed-1], PalSort[X]);
            IF OneDist < ClosestDist THEN
              begin
                ClosestDist := OneDist;
                CloseX  := X;
              end;
          end;
        // add the last item's pixel-count to the closest one
        Inc(PalSort[CloseX].Pixels, PalSort[SlotsUsed-1].Pixels);
        // zero out the last item's pixel-count
        PalSort[SlotsUsed-1].Pixels := 0;
        // set the last item to be replaced by the closest
        PalSort[SlotsUsed-1].NewIndex := PalSort[CloseX].OrgIndex;
        // the closest item now has more pixels; to maintain the
        // sorted order it may need to move lower in the list
        X := CloseX-1;
        WHILE (X > 0) AND
          (PalSort[X].Pixels < PalSort[CloseX].Pixels) DO Dec(X);
        Inc(X);
        IF PalSort[X].Pixels < PalSort[CloseX].Pixels THEN
          begin
            SwapTemp        := PalSort[CloseX];
            PalSort[CloseX] := PalSort[X];
            PalSort[X]      := SwapTemp;
          end;
        // some of the now-unused colors may have their new index
        // set to the color we just replaced; change them to point
        // to the color by which it was replaced
        FOR X := SlotsUsed TO 255 DO
          IF PalSort[X].NewIndex = PalSort[SlotsUsed-1].OrgIndex THEN
            PalSort[X].NewIndex := PalSort[CloseX].OrgIndex;
        Dec(SlotsUsed);
      end;
  end;

begin
  // if this is a foreground bitmap, the lower-left pixel defines
  // the "transparent" color
  IF IsFore THEN
    TransColorSlot := GetTransColorSlot(BS.TheFilename)
  ELSE TransColorSlot := -1;
  // Initialize the palette-manipulation structure
  FOR N := 0 TO 255 DO
    WITH PalSort[N], BS.BIPalette[N] DO
      begin
        NewIndex  := N;
        OrgIndex  := N;
        pstRed    := rgbRed;
        pstGreen  := rgbGreen;
        pstBlue   := rgbBlue;
        Pixels    := BS.PixelCount[N];
      end;
  // combine entries that are the exact same color
  FOR N := 0 TO 254 DO
    FOR M := N+1 TO 255 DO
      IF EquColor(PalSort[N], PalSort[M]) AND
        (PalSort[N].Pixels > 0) AND (PalSort[M].Pixels>0) THEN
        begin
          Inc(PalSort[N].Pixels, PalSort[M].Pixels);
          PalSort[M].Pixels := 0;
          PalSort[M].NewIndex := N;
        end;
  SortByPrevalence;
  SlotMax := ColorsToUse;
  SlotsUsed := 0;
  FOR N := 0 TO 255 DO
    IF PalSort[N].Pixels>0 THEN
      Inc(SlotsUsed);
  ReduceColors;
  // Now store the array order and restore original order
  FillChar(BS.PixelCount, SizeOf(BS.PixelCount), 0);
  FOR N := 0 TO SlotsUsed-1 DO
    WITH PalSort[N] DO
      begin
        NewIndex := N;
        BS.PixelCount[N] := Pixels;
      end;
  FOR N := SlotsUsed TO 255 DO
    begin
      IF PalSort[N].NewIndex <> PalSort[N].OrgIndex THEN
        FOR M := 0 TO SlotsUsed-1 DO
          IF PalSort[N].NewIndex = PalSort[M].OrgIndex THEN
            begin
              PalSort[N].NewIndex := M;
              break;
            end;
    end;
  FillChar(NuFilePalette, SizeOf(NuFilePalette), 0);
  FOR N := 0 TO SlotsUsed-1 DO
    WITH NuFilePalette[N], PalSort[N] DO
      begin
        rgbRed   := pstRed;
        rgbGreen := pstGreen;
        rgbBlue  := pstBlue;
      end;
  SortOnOrgIndex;
  FOR N := 0 TO 255 DO
    TransTable[N] := PalSort[N].NewIndex;
  TempFilename := Copy(BS.TheFilename, 1, Length(BS.TheFilename)-
    Length(ExtractFileExt(BS.TheFilename))) + '_temp.BMP';
  IF FileExists(TempFilename) THEN DeleteFile(TempFilename);
  AssignFile(F1, BS.TheFilename);
  AssignFile(F2, TempFilename);
  Reset(F1, 1);
  ReWrite(F2, 1);
  try
    BlockWrite(F2, BS.BFH, SizeOf(BS.BFH));
    WITH BS.BIH DO
      begin
        biXPelsPerMeter := 3790;
        biYPelsPerMeter := 3780;
      end;
    BlockWrite(F2, BS.BIH, SizeOf(BS.BIH));
    Seek(F1, BS.BFH.bfOffBits);
    BlockWrite(F2, NuFilePalette, SizeOf(NuFilePalette));
    FOR N := 0 TO 399 DO
      begin
        BlockRead(F1, PixelRow, SizeOf(PixelRow));
        FOR M := 0 TO 319 DO
          PixelRow[M] := TransTable[PixelRow[M]];
        BlockWrite(F2, PixelRow, SizeOf(PixelRow));
      end;
  finally
    CloseFile(F1);
    CloseFile(F2);
  end;
  Move(NuFilePalette, BS.BIPalette, SizeOf(BS.BIPalette));
  IF FileExists(S) THEN DeleteFile(S);
  RenameFile(TempFileName, S);
  BS.TheFilename := S;
  UpdateBitmapStuff(BS);
end;

function TMainForm.AppOnHelp(Command: Word; Data: Longint;
  VAR CallHelp : Boolean): Boolean;
begin
  Result := False;
  IF Command = HELP_CONTEXTPOPUP THEN
    begin
      IF KeyWasF1 THEN
        begin
          KeyWasF1 := False;
          CallHelp := False;
          Result := Application.HelpCommand(HELP_CONTEXT,
            Screen.ActiveForm.HelpContext);
        end
      ELSE
        CASE Data OF
          ch_btnChoose..ch_BtnCancel,
          ch_rgShape..ch_btnLogoDetails : CallHelp := True;
          ELSE
            BEGIN
              CallHelp := False;
              Result := Application.HelpCommand(HELP_CONTEXT, Data);
            END
        END;
    end;
end;

// **** Regular Delphi event handler methods
procedure TMainForm.FormCreate(Sender: TObject);
VAR
  TDM          : TDevMode;
  CanDo640x400 : Boolean;
begin
  WITH TIniFile.Create(IniName) DO
    try
      CASE ReadInteger('Settings', 'CanDo640x400', 0) OF
        1 : CanDo640x400 := False;
        2 : CanDo640x400 := True;
        ELSE
          begin
            TDM.dmSize       := SizeOf(TDM);
            EnumDisplaySettings(NIL, 0, TDM);
            TDM.dmPelsWidth  := 640;
            TDM.dmPelsHeight := 400;
            TDM.dmFields     := DM_PELSHEIGHT OR DM_PELSWIDTH;
            CanDo640x400 := ChangeDisplaySettings(@TDM, CDS_TEST) =
              DISP_CHANGE_SUCCESSFUL;
            WriteInteger('Settings', 'CanDo640x400',
              Succ(Ord(CanDo640x400)));
          end;
      END;
    finally
      free;
    end;
  CanPreview := (GetDeviceCaps(Canvas.Handle, BITSPIXEL)=8) AND
    (GetDeviceCaps(Canvas.Handle, RASTERCAPS) AND RC_PALETTE >0) AND
    (GetDeviceCaps(Canvas.Handle, SIZEPALETTE)=256);
  IF NOT (CanPreview AND CanDo640x400) THEN
    cbFullScreen.Enabled := False;
  Application.HelpFile := ChangeFileExt(Application.ExeName, '.HLP');
  Application.OnHelp := AppOnHelp;
  PageControl1.ActivePage := tsAnimation;
  WITH StatBitmapStuff DO
    begin
      lblFilename  := lblStatName;
      lblSize      := lblStatSize;
      lblColors    := lblStatColors;
      btnDetails   := btnStatDetails;
      btnResize    := btnStatResize;
      btnCenter    := btnStatCenter;
      cbForeground := cbStatForeground;
      cbOtherFore  := cbAnimForeground;
      imgThumbnail := imgStat;
    end;
  WITH AnimBitmapStuff DO
    begin
      lblFilename  := lblAnimName;
      lblSize      := lblAnimSize;
      lblColors    := lblAnimColors;
      btnDetails   := btnAnimDetails;
      btnResize    := btnAnimResize;
      btnCenter    := btnAnimCenter;
      cbForeground := cbAnimForeground;
      cbOtherFore  := cbStatForeground;
      imgThumbnail := imgAnim;
    end;
  WITH LogoBitmapStuff DO
    begin
      lblFilename  := nil;
      lblSize      := nil;
      lblColors    := lblLogoColors;
      btnDetails   := btnLogoDetails;
      btnResize    := nil;
      btnCenter    := nil;
      cbForeground := nil;
      cbOtherFore  := nil;
      imgThumbnail := nil;
    end;
  btnStatChoose.Tag    := Integer(@StatBitmapStuff);
  btnStatDetails.Tag   := Integer(@StatBitmapStuff);
  btnStatResize.Tag    := Integer(@StatBitmapStuff);
  btnStatCenter.Tag    := Integer(@StatBitmapStuff);
  cbStatForeground.Tag := Integer(@StatBitmapStuff);
  btnAnimChoose.Tag    := Integer(@AnimBitmapStuff);
  btnAnimDetails.Tag   := Integer(@AnimBitmapStuff);
  btnAnimResize.Tag    := Integer(@AnimBitmapStuff);
  btnAnimCenter.Tag    := Integer(@AnimBitmapStuff);
  cbAnimForeground.Tag := Integer(@AnimBitmapStuff);
  btnLogoDetails.Tag   := Integer(@LogoBitmapStuff);
  PositionFromINIFile(Self);
end;

procedure TMainForm.PreviewClick(Sender: TObject);
begin
  // Let N = animcolors+statcolors-238. If N > 0 then the top
  // N colors in the animation portion will show as black in
  // the preview mode.
  WITH TPreviewForm.Create(Self) DO
    try
      SetFullScreen(cbFullScreen.Checked);
      SetBitmap(logoBitmapStuff.TheFilename);
      ShowModal;
    finally
      Free;
      Self.Refresh;
    end;
end;

procedure TMainForm.btnChooseClick(Sender: TObject);
VAR
  BS        : PBitmapStuff;
  OKFile    : Boolean;
  OtherFile : String;
begin
  BS := PBitmapStuff((Sender AS TBitBtn).Tag);
  IF BS = @AnimBitmapStuff THEN
    OtherFile := StatBitmapStuff.TheFilename
  ELSE OtherFile := AnimBitmapStuff.TheFilename;
  OKFile := False;
  WITH DM1.OpenDialog1 DO
    IF Execute THEN
      IF ofExtensionDifferent IN Options THEN
        ErrorMsg('Extension must be .BMP')
      ELSE IF SameFile(Filename, OtherFile) THEN
        ErrorMsg('You cannot use the same file for the '+
          'animation bitmap and the static bitmap')
      ELSE
        begin
          ReadAndRateBitmap(Filename, BS^);
          UpdateBitmapStuff(BS^);
          OKFile := BS^.BitmapType <> lgNone;
        end;
  IF NOT OkFile THEN
    begin
      BS^.TheFilename := '';
      UpdateBitmapStuff(BS^);
    end;
end;

procedure TMainForm.btnAnimClick(Sender: TObject);
begin
  WITH TBitmapInfoForm.Create(Self) DO
    try
      WITH PBitmapStuff((Sender AS TBitBtn).Tag)^ DO
        Initialize(TheFilename, BFH, BIH, ColorsInUse);
      ShowModal;
    finally
      Free;
    end;
end;

procedure TMainForm.btnResizeClick(Sender: TObject);
VAR BS : PBitmapStuff;
begin
  BS := PBitmapStuff((Sender AS TBitBtn).Tag);
  WITH DM1.SaveDialog1 DO
    begin
      Filename := '';
      Title := 'Save tiled/chopped to 320x400';
      IF Execute THEN
        IF ofExtensionDifferent IN Options THEN
          ErrorMsg('Extension must be .BMP')
        ELSE ProcrustesBed(BS^, Filename)
    end;
end;

procedure TMainForm.cbStatForegroundClick(Sender: TObject);
VAR BS : PBitmapStuff;
begin
  BS := PBitmapStuff((Sender AS TCheckBox).Tag);
  BS.cbOtherFore.Checked := NOT BS.cbForeground.Checked;
end;

procedure TMainForm.btnFewerColorsClick(Sender: TObject);
begin
  WITH DM1.SaveDialog1 DO
    begin
      Filename := '';
      Title := Format('Save using %d colors', [udColors.Position]);
      IF Execute THEN
        IF ofExtensionDifferent IN Options THEN
          ErrorMsg('Extension must be .BMP')
        ELSE SaveWithReducedColors(StatBitmapStuff, Filename,
          udColors.Position, cbStatForeground.Checked);
    end;
end;

procedure TMainForm.btnCreateClick(Sender: TObject);
// only enabled if both animated and static bitmaps are 320x400 AND
// the sum of the used colors is less than or equal to 256
begin
  WITH DM1.SaveDialog2 DO
    begin
      Filename := '';
      IF Execute THEN
        IF ofExtensionDifferent IN Options THEN
          ErrorMsg('Extension must be .SYS')
        ELSE CreateLogoFile(Filename);
    end;
end;

procedure TMainForm.btnLoadClick(Sender: TObject);
begin
  WITH DM1.OpenDialog2 DO
    begin
      Filename := '';
      IF Execute THEN
        IF ofExtensionDifferent IN Options THEN
          ErrorMsg('Extension must be .SYS')
        ELSE
          begin
            ReadAndRateBitmap(Filename, logoBitmapStuff);
            IF logoBitmapStuff.BitmapType = lgSYSa THEN
              WITH logoBitmapStuff DO
                begin
                  CountThePixelsIn(logoBitmapStuff);
                  lblLogoName.Caption := ExtractFileName(TheFilename);
                  lblLogoColors.Caption := Format('Uses %d colors',
                    [ColorsInUse]);
                  imgLogo.Picture.Bitmap.LoadFromFile(TheFilename);
                  btnPreview.Enabled := CanPreview;
                  btnLogoDetails.Enabled := True;
                end
            ELSE
              ErrorMsg(Format('Sorry, %s is not a Win95 animated '+
                'logo bitmap', [Filename]));
          end;
    end;
end;

procedure TMainForm.btnPaletteClick(Sender: TObject);
VAR F : File;
begin
  WITH TPalForm.Create(Self) DO
    try
      WITH animBitmapStuff DO
        begin
          Initialize(BIPalette, PixelCount, ColorsInUse);
          IF ShowModal = mrOK THEN
            begin
              AssignFile(F, TheFilename);
              Reset(F, 1);
              Seek(F, SizeOf(BFH) + SizeOf(BIH));
              BlockWrite(F, BIPalette, SizeOf(BIPalette));
              CloseFile(F);
              imgAnim.Picture.Bitmap.LoadFromFile(TheFilename);
            end;
        end;
    finally
      Free;
    end;
end;

procedure TMainForm.btnHelpClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  KeyWasF1 := (Key = VK_F1) AND (Shift=[]);
end;

procedure TMainForm.PageControl1Change(Sender: TObject);
begin
  HelpContext := (Sender AS TPageControl).ActivePage.HelpContext;
  (Sender AS TPageControl).HelpContext := HelpContext;
end;

procedure TMainForm.btnAnimCenterClick(Sender: TObject);
VAR BS : PBitmapStuff;
begin
  BS := PBitmapStuff((Sender AS TBitBtn).Tag);
  WITH DM1.SaveDialog1 DO
    begin
      Filename := '';
      Title := 'Save centered in 320x400';
      IF Execute THEN
        IF ofExtensionDifferent IN Options THEN
          ErrorMsg('Extension must be .BMP')
        ELSE SaveCentered(BS^, Filename)
    end;
end;

procedure TMainForm.btnGenerateClick(Sender: TObject);
begin
  WITH GenForm DO
    begin
      GenFilename := ExtractFilePath(Application.ExeName) +
        '\LogoMania_gen.bmp';
      IF ShowModal = mrOK THEN
        begin
          ReadAndRateBitmap(GenFilename, animBitmapStuff);
          UpdateBitmapStuff(animBitmapStuff);
          cbStatForeground.Checked    := True;
        end;
    end;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Application.HelpCommand(HELP_QUIT, 0);
  PositionToIniFile(Self);
end;

end.
