unit logopalu;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, StdCtrls, Buttons, ComCtrls, LogoAllU, ExtCtrls;

type
  TPalForm = class(TForm)
    Panel1: TPanel;
    lblIndex: TLabel;
    dgPalette: TDrawGrid;
    edRed: TEdit;
    edGreen: TEdit;
    edBlue: TEdit;
    udRed: TUpDown;
    udGreen: TUpDown;
    udBlue: TUpDown;
    btnChangeBmp: TBitBtn;
    btnCancel: TBitBtn;
    btnCopyAbove: TBitBtn;
    procedure StringGrid1DrawCell(Sender: TObject; vCol, vRow:
      Longint; Rect: TRect; State: TGridDrawState);
    procedure dgPaletteSelectCell(Sender: TObject; vCol, vRow:
      Longint; var CanSelect: Boolean);
    procedure edRedChange(Sender: TObject);
    procedure btnChangeBmpClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure btnCopyAboveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    pFP  : PFilePaletteType;
    FP   : TFilePaletteType;
    PC   : TPixelCountType;
    TTbl : ARRAY[0..255] OF Byte;
  public
    { Public declarations }
    procedure Initialize(VAR vFP : TFilePaletteType;
      const vPC : TPixelCountType; NumUsed : Integer);
  end;

var
  PalForm: TPalForm;

implementation

{$R *.DFM}

procedure TPalForm.Initialize(VAR vFP : TFilePaletteType;
  const vPC : TPixelCountType; NumUsed : Integer);
VAR
  N, M         : Integer;
  dummyBoolean : Boolean;
begin
  pFP := @vFP;
  PC := vPC;
  FP := pFP^;
  dgPalette.RowCount := NumUsed+1;
  M := 0;
  FOR N := 0 TO 255 DO
    IF PC[N] > 0 THEN
      begin
        TTbl[M] := N;
        Inc(M);
      end;
  dgPaletteSelectCell(dgPalette, 1, 1, dummyBoolean);
end;


procedure TPalForm.StringGrid1DrawCell(Sender: TObject; vCol, vRow:
  Longint; Rect: TRect; State: TGridDrawState);
begin
  WITH (Sender AS TDrawGrid), Canvas DO
    begin
      IF vRow=0 THEN
        CASE vCol OF
          0 : TextRect(Rect, Rect.Left+1, Rect.Top+1, 'Index');
          1 : TextRect(Rect, Rect.Left+1, Rect.Top+1, 'Red');
          2 : TextRect(Rect, Rect.Left+1, Rect.Top+1, 'Green');
          3 : TextRect(Rect, Rect.Left+1, Rect.Top+1, 'Blue');
          4 : TextRect(Rect, Rect.Left+1, Rect.Top+1, 'Color');
        END
      ELSE
        begin
          CASE vCol OF
            0 : TextRect(Rect, Rect.Left+1, Rect.Top+1,
                  IntToStr(TTbl[vRow-1]));
            1 : begin
              IF pFP^[TTbl[vRow-1]].rgbRed =
                   FP[TTbl[vRow-1]].rgbRed THEN
                  Font.Style := []
              ELSE Font.Style := [fsBold];
              TextRect(Rect, Rect.Left+1, Rect.Top+1,
                IntToStr(FP[TTbl[vRow-1]].rgbRed));
            end;
            2 : begin
              IF pFP^[TTbl[vRow-1]].rgbGreen =
                   FP[TTbl[vRow-1]].rgbGreen THEN
                  Font.Style := []
              ELSE Font.Style := [fsBold];
              TextRect(Rect, Rect.Left+1, Rect.Top+1,
                IntToStr(FP[TTbl[vRow-1]].rgbGreen));
            end;
            3 : begin
              IF pFP^[TTbl[vRow-1]].rgbBlue =
                   FP[TTbl[vRow-1]].rgbBlue THEN
                  Font.Style := []
              ELSE Font.Style := [fsBold];
              TextRect(Rect, Rect.Left+1, Rect.Top+1,
                IntToStr(FP[TTbl[vRow-1]].rgbBlue));
            end;
            4 : begin
              WITH FP[TTbl[vRow-1]] DO
                Brush.Color := RGB(rgbRed, rgbGreen, rgbBlue);
              FillRect(Rect);
            end;
          END
        end;
    end;
end;

procedure TPalForm.dgPaletteSelectCell(Sender: TObject; vCol, vRow:
  Longint; VAR CanSelect: Boolean);
begin
  btnCopyAbove.Enabled := vRow > 1;
  lblIndex.Caption := IntToStr(TTbl[vRow-1]);
  edRed.OnChange   := NIL;
  edGreen.OnChange := NIL;
  edBlue.OnChange  := NIL;
  edRed.Text       := IntToStr(FP[TTbl[vRow-1]].rgbRed);
  edGreen.Text     := IntToStr(FP[TTbl[vRow-1]].rgbGreen);
  edBlue.Text      := IntToStr(FP[TTbl[vRow-1]].rgbBlue);
  edRed.OnChange   := edRedChange;
  edGreen.OnChange := edRedChange;
  edBlue.OnChange  := edRedChange;
end;

procedure TPalForm.edRedChange(Sender: TObject);
VAR Idx, Valu : Integer;
begin
  Valu := StrToIntDef((Sender AS TEdit).Text, 0);
  IF (Valu < 0) OR (Valu > 255) THEN Exit;
  Idx := TTbl[dgPalette.Selection.Top-1];
  CASE (Sender AS TEdit).Tag OF
    1 : FP[Idx].rgbRed   := Valu;
    2 : FP[Idx].rgbGreen := Valu;
    3 : FP[Idx].rgbBlue  := Valu;
  END;
  dgPalette.Refresh;
end;

procedure TPalForm.btnChangeBmpClick(Sender: TObject);
begin
  pFP^ := FP;
  ModalResult := mrOK;
end;

procedure TPalForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  KeyWasF1 := (Key = VK_F1) AND (Shift=[]);
end;

procedure TPalForm.btnCopyAboveClick(Sender: TObject);
VAR
  Idx1, Idx2 : Integer;
  R          : TGridRect;
begin
  Idx1 := TTbl[dgPalette.Selection.Top-2];
  Idx2 := TTbl[dgPalette.Selection.Top-1];
  FP[Idx2].rgbRed   := FP[Idx1].rgbRed;
  FP[Idx2].rgbGreen := FP[Idx1].rgbGreen;
  FP[Idx2].rgbBlue  := FP[Idx1].rgbBlue;
  R := dgPalette.Selection;
  Inc(R.Top); Inc(R.Bottom);
  WITH dgPalette DO
    begin
      Selection := R;
      IF TopRow < RowCount - VisibleRowCount THEN
      TopRow := TopRow+1;
      Refresh;
    end;
end;

procedure TPalForm.FormCreate(Sender: TObject);
begin
  PositionFromIniFile(Self);
end;

procedure TPalForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  PositionToIniFile(Self);
end;

end.

